/* ui-commit.c: generate commit view
 *
 * Copyright (C) 2006-2014 cgit Development Team <cgit@lists.zx2c4.com>
 *                    2018 Vincenzo 'KatolaZ' Nicosia <katolaz@freaknet.org>
 *
 * Licensed under GNU General Public License v2
 *   (see COPYING for full license text)
 */

#include "cgit.h"
#include "ui-commit.h"
#include "html.h"
#include "ui_70-shared.h"
#include "ui-diff.h"
#include "ui-log.h"

void cgit_print_commit(char *hex, const char *prefix)
{
	struct commit *commit, *parent;
	struct commitinfo *info, *parent_info;
	struct commit_list *p;
	struct strbuf notes = STRBUF_INIT;
	struct object_id oid;
	char *tmp, *tmp2;
	int parents = 0;

	if (!hex)
		hex = ctx.qry.head;

	if (get_oid(hex, &oid)) {
		cgit_gopher_error("Bad object id");
		return;
	}
	commit = lookup_commit_reference(&oid);
	if (!commit) {
		cgit_gopher_error("Bad commit reference");
		return;
	}
	info = cgit_parse_commit(commit);

	/*format_display_notes(&oid, &notes, PAGE_ENCODING, 0);*/

	load_ref_decorations(NULL, DECORATE_FULL_REFS);

	cgit_print_layout_start();
	/*cgit_print_diff_ctrls();*/

	cgit_gopher_start_selector(GOPHER_INFO);
	cgit_gopher_text("author: ");
	cgit_gopher_text(info->author);
	if (!ctx.cfg.noplainemail) {
		cgit_gopher_text(" ");
		cgit_gopher_text(info->author_email);
		cgit_gopher_text(" ");
	}
	cgit_gopher_text(show_date(info->author_date, info->author_tz,
				cgit_date_mode(DATE_ISO8601)));
	cgit_gopher_tab();
	cgit_gopher_selector_link("Err");
	cgit_gopher_end_selector();

	cgit_gopher_start_selector(GOPHER_INFO);
	cgit_gopher_text("committer: ");
	cgit_gopher_text(info->committer);
	if (!ctx.cfg.noplainemail) {
		cgit_gopher_text(" ");
		cgit_gopher_text(info->committer_email);
		cgit_gopher_text(" ");
	}
	cgit_gopher_text(show_date(info->committer_date, info->committer_tz,
				cgit_date_mode(DATE_ISO8601)));
	cgit_gopher_tab();
	cgit_gopher_selector_link("Err");
	cgit_gopher_end_selector();
	
	cgit_gopher_start_selector(GOPHER_MENU);
	cgit_gopher_text("commit ");
	tmp = oid_to_hex(&commit->object.oid);
	cgit_gopher_text(tmp);
	cgit_gopher_tab();
	cgit_commit_link(tmp, NULL, NULL, ctx.qry.head, tmp, prefix);
	cgit_gopher_end_selector();

	cgit_gopher_start_selector(GOPHER_TXT);
	cgit_gopher_text("patch ");
	cgit_gopher_tab();
	cgit_patch_link("patch", NULL, NULL, NULL, tmp, prefix);
	cgit_gopher_end_selector();
	
	cgit_gopher_start_selector(GOPHER_MENU);
	cgit_gopher_text("tree ");
	cgit_gopher_tab();
	tmp = xstrdup(hex);
	cgit_tree_link(oid_to_hex(&commit->maybe_tree->object.oid), NULL, NULL,
		       ctx.qry.head, tmp, NULL);
	cgit_gopher_end_selector();

/*	if (prefix) {
		html(" /");
		cgit_tree_link(prefix, NULL, NULL, ctx.qry.head, tmp, prefix);
	}
*/	free(tmp);
	
	for (p = commit->parents; p; p = p->next) {
		parent = lookup_commit_reference(&p->item->object.oid);
		if (!parent) {
			cgit_gopher_info("Error reading parent commit");
			continue;
		}
		
		cgit_gopher_start_selector(GOPHER_MENU);
		cgit_gopher_text("parent ");
		cgit_gopher_tab();
		tmp = tmp2 = oid_to_hex(&p->item->object.oid);
		if (ctx.repo->enable_subject_links) {
			parent_info = cgit_parse_commit(parent);
			tmp2 = parent_info->subject;
		}
		cgit_commit_link(tmp2, NULL, NULL, ctx.qry.head, tmp, prefix);
		cgit_gopher_end_selector();
		/*cgit_diff_link("diff", NULL, NULL, ctx.qry.head, hex,
			       oid_to_hex(&p->item->object.oid), prefix);
		html(")</td></tr>");
		*/
		parents++;
	}
	/*
	if (ctx.repo->snapshots) {
		html("<tr><th>download</th><td colspan='2' class='sha1'>");
		cgit_print_snapshot_links(ctx.repo, hex, "<br/>");
		html("</td></tr>");
	}
	*/
	cgit_gopher_start_selector(GOPHER_INFO);
	cgit_gopher_text("subject: ");
	cgit_gopher_text(info->subject);
	cgit_gopher_tab();
	cgit_gopher_selector_link("Err");
	cgit_gopher_end_selector();

	if (strlen(info->msg)){	
		cgit_gopher_start_selector(GOPHER_INFO);
		cgit_gopher_text("message: ");
		cgit_gopher_text(info->msg);
		cgit_gopher_tab();
		cgit_gopher_selector_link("Err");
		cgit_gopher_end_selector();
	}
	
	/* FIXME: NOTES HAVE BEEN DISABLED*/	
/*	if (notes.len != 0) {
		html("<div class='notes-header'>Notes</div>");
		html("<div class='notes'>");
		cgit_open_filter(ctx.repo->commit_filter);
		html_txt(notes.buf);
		cgit_close_filter(ctx.repo->commit_filter);
		html("</div>");
		html("<div class='notes-footer'></div>");
	}*/
/*	if (parents < 3) {
		if (parents)
			tmp = oid_to_hex(&commit->parents->item->object.oid);
		else
			tmp = NULL;
		cgit_print_diff(ctx.qry.sha1, tmp, prefix, 0, 0);
	}
*/
	strbuf_release(&notes);
	cgit_free_commitinfo(info);
	cgit_print_layout_end();
}
